.pausedOverlay {
  background-color:#babdb6;
  color: white;
  font-size:32px;
  border-radius: 3px;
}

.pausedOverlay:backdrop {
  background-color: shade(#babdb6,1.1);
}

.count, .explodedField .tile {
-gtk-icon-effect: none;
}

.navigation {
  border-radius:32px;
  background-color:alpha(#888a85, 0.1);
  border-width:0px;
  border-color:#BABABA;
  background-image:none;
}

/* These are the default style settings possibly overridden by themes */

/* Common style for all tiles */
.tile {
  background-image:none;
  background-color:#babdb6;
  border-color:@theme_bg_color;
  border-radius:4px; /* border width might be affecting */
  color:transparent;
  border-image:none;
  font-size:16px;
  font-weight: bold;
  background-size: 60%;
  background-position: 50%;
  padding:0px;
  background-repeat: no-repeat;
  box-shadow:none;
  border-width:2px;
}

.tile:not(.cursor):hover {
  background-color:#d3d7cf;
  transition-duration:0.4s;
}

.tile:active {
  background-color:#888a85;
  box-shadow: inset 0 2px 1px rgba(0,0,0,.2);
}

/* Style for revealed fields showing numbers */
.count {
  color:black;
  font-size:0px;
  background-image:none;
  background-color:#dededc;
  transition-duration:0.4s;
}

.count:not(.cursor):hover {
  background-color:#dededc;
}

/* Style for mines */
.mine {
  background-color:#729fcf;
}

/* Style of the keyboard cursor */
.cursor {
  background-color:#e9b96e;
}

/* Styles to be used on game over*/
.explodedField .incorrect {
  background-color:#cc0000;
}

.completedField .flag {
  transition-duration:1s;
}

.explodedField {
  transition-duration:1s;
}

