/* Style of the unrevealed tiles after game over */
.explodedField .mine {
  background-color:#888a85;
}

.explodedField .exploded {
  background-color:#d3d7cf;
}

.1mines {
  background-color:#DDFAC3;
}

.2mines {
  background-color:#ECEDBF;
}

.3mines {
  background-color:#EDDAB4;
}

.4mines {
  background-color:#EDC38A;
}

.5mines {
  background-color:#F7A1A2;
}
.6mines {
  background-color:#FEA785;
}

.7mines {
  background-color:#FF7D60;
}

.8mines {
  background-color:#FF323C;
}
